(function ($, Drupal, once, drupalSettings, cloudinary) {
  Drupal.behaviors.CloudinaryMediaLibraryWidget = {
    attach: function (context) {
      const settings = drupalSettings.cloudinary_media_library_widget || false;

      if (!settings) {
        console.error('No settings available for the widget.');
        return;
      }

      const cloudinary_settings = {
        cloud_name: settings.cloud_name,
        api_key: settings.api_key,
        button_caption: Drupal.t('Open media library', {}, {context: 'Cloudinary media'}),
        button_class: 'button cloudinary-media-library-button',
        insert_caption: Drupal.t('Insert', {}, {context: 'Cloudinary media'}),
        multiple: settings.multiple,
        use_saml: settings.use_saml || false,
        folder: {
          resource_type: settings.resource_type
        }
      };

      if (settings.max_files) {
        cloudinary_settings.max_files = settings.max_files;
      }

      if (settings.starting_folder) {
        cloudinary_settings.folder.path = settings.starting_folder;
      }

      const media_creation_url = 'cloudinary_media_library_widget/' + settings.bundle;
      const $progress_element = $(Drupal.theme('ajaxProgressThrobber'));

      let mediaInsertHandler = function (data) {
        const $input_element = $('#' + this.input_id);
        const $preview_element = $input_element.closest('.cloudinary-media').find('.js-cloudinary-media--preview');
        const $remove_element = $input_element.closest('.cloudinary-media').find('.js-cloudinary-media--remove');

        $preview_element.append($progress_element);
        $remove_element.hide();

        data.assets.forEach(
          asset => {
            $.ajax({
              url: Drupal.url(media_creation_url),
              type: 'POST',
              data: JSON.stringify(asset),
              dataType: 'json',
              success: function (response) {
                $input_element.val(response.id);
                $preview_element.html(response.preview);
                $remove_element.show();
              }
            });
          })
      }

      once('cloudinaryMediaLibraryWidget', '.js-cloudinary-media--input', context).forEach(function (element) {
        const widget = cloudinary.createMediaLibrary(
          cloudinary_settings,
          {
            insertHandler: mediaInsertHandler
          },
          element
        )

        widget.input_id = element.id;
      });

      once('cloudinaryMediaLibraryRemove', '.js-cloudinary-media--remove', context).forEach(function (element) {
        // Hide "remove" link if the media has not been selected.
        if (!$(element).closest('.cloudinary-media').find('.js-cloudinary-media--input').val().length) {
          $(element).hide();
        }

        $(element).on('click', function (e) {
          e.preventDefault();

          const $formWrapper = $(this).closest('.cloudinary-media');

          $formWrapper.find('.js-cloudinary-media--input').val('');
          $formWrapper.find('.js-cloudinary-media--preview').html('');

          $(this).hide();
        })
      });
    }
  };
})(jQuery, Drupal, once, drupalSettings, cloudinary);
