<?php

namespace Drupal\cloudinary_media_library_widget\Controller;

use Drupal\cloudinary_media_library_widget\Model\Asset;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Media controller.
 */
class MediaController extends ControllerBase {

  /**
   * The serializer which serializes the views result.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * The Cloudinary media manager.
   *
   * @var \Drupal\cloudinary_media_library_widget\CloudinaryMediaManagerInterface
   */
  protected $cloudinaryMediaManager;

  /**
   * The render service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->serializer = $container->get('serializer');
    $instance->cloudinaryMediaManager = $container->get('cloudinary_media_library_widget.manager.cloudinary_media');
    $instance->renderer = $container->get('renderer');

    return $instance;
  }

  /**
   * Returns Ajax Response containing the media id and preview.
   *
   * @param string $bundle
   *   Media bundle type.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The json response.
   */
  public function createMedia(Request $request, string $bundle): JsonResponse {
    $asset_data = Json::decode($request->getContent());

    if (is_null($asset_data)) {
      throw new \InvalidArgumentException('Assert data have not been provided');
    }

    /** @var \Drupal\cloudinary_media_library_widget\Model\Asset $asset */
    $asset = $this->serializer->denormalize($asset_data, Asset::class);

    $cloudinary_media_plugin = $this->cloudinaryMediaManager->createInstanceByType($asset->getResourceType());
    $media = $cloudinary_media_plugin->createMedia($asset, $bundle);
    $preview = $cloudinary_media_plugin->getPreview($media);

    return new JsonResponse([
      'id' => $media->id(),
      'preview' => $this->renderer->render($preview),
    ]);
  }

}
